Copyright (C) 2020-2022 MicroSys Electronics GmbH

# Yocto Build Setup

This text describes how to setup a Yocto build environment for
the s32g274asbc2 board.

## Download and Clone NXP's Yocto Repository

### Presumptions

BSP32.0 is based on Yocto Poky 3.2.4.

You need a build host that is supported by this Yocto release.

Please check the documentation at
[System Requirements Poky 3.2.4](https://docs.yoctoproject.org/3.2.4/ref-manual/ref-system-requirements.html).

You might also follow the instructions in the  _Linux BSP 32.0 User Manual_  from NXP.

For first time setup NXP recommends to execute:<br>
    `# ./sources/meta-alb/scripts/host-prepare.sh`

### Steps

    1. Create and go to a directory of your choice
    2. Install 'repo' if not already done:
         # curl https://commondatastorage.googleapis.com/git-repo-downloads/repo > /usr/local/bin/repo
         # chmod a+x /usr/local/bin/repo
    3. Execute the following commands:
         # repo init --no-clone-bundle -u https://source.codeaurora.org/external/autobsps32/auto_yocto_bsp -b release/bsp32.0
         # repo sync --force-sync --no-clone-bundle
       After executing these two commands you should have a directory called sources/
       and a script called nxp-setup-alb.sh in your current directory.

## Install MicroSys' Layer for S32G

    1. Go to the directory sources/ and copy or unpack the layer
       'meta-microsys-auto' there. You should then have a directory
       with the layer name in sources/.

       Note: the name of the top directory in the TAR-ball or ZIP-file could contain
       a revision in its name. In this case rename it to 'meta-microsys-auto'.
    2. Source the script nxp-setup-alb.sh:
         # . nxp-setup-alb.sh -e meta-microsys-auto -m s32g274asbc2 -D microsys-auto
       You are now ready for starting to build the s32g274asbc2 image.
    3. Start the build for the image 'microsys-image-auto':
         # bitbake microsys-image-auto

## Program QSPI Flash with Bootloader

    1. Boot from SD-card into the U-Boot prompt
    2. Program the firmware image with:
         => run flash
    3. Switch to QSPI boot media
       This can be done via using the onboard DIP-switch or
       you re-program the RCW in the EEPROM with:
         => run rcwqspi

Note that you probably have to update your U-Boot environment before performing these
steps:

     => env default -f flash
     => env default -f flash_img
     => env delete kconfig
     => env save

## Program the PFE Firmware into QSPI

    1. Boot from SD-card into the U-Boot prompt
    2. Program the firmware image with:
         => run flashfw

Note that you probably have to update your U-Boot environment before performing these
steps:

     => env default -f flashfw
     => env default -f flashfw_img
     => env default -f pfengfw
     => env delete kconfig
     => env save


# CAN Busses

The boards s32g274asbc and s32g274asbc2/3 have 18 CAN busses routed to external connectors.

## Interfaces

<table border="1px solid black">
    <tr><th>Linux CAN Interface</th><th>S32G Pins</th></tr>
    <tr><td>can0</td><td>CAN2</td></tr>
    <tr><td>can1</td><td>CAN3</td></tr>
    <tr><td>llcecan0</td><td>LLCE CAN0</td></tr>
    <tr><td>...</td><td>...</td></tr>
    <tr><td>llcecan15</td><td>LLCE CAN15</td></tr>
</table>

## Configuration

The interfaces can be configured with the 'ip' command.

For `can0` and `can1` the following commands bring-up the interfaces:

  `# ip link set can0 up type can bitrate 1000000`

  `# ip link set can1 up type can bitrate 1000000`

The LLCE CAN interfaces need, e.g. for `llcecan0`:

  `# ip link set llcecan0 up type can bitrate 500000 dbitrate 5000000 fd on`

# SJA1110 TSN Switch

The platforms S32G274ASBC2/3 are equipped with a SJA1110 TSN switch.
The Linux kernel automatically uploads a fitting firmware image into the SJA1110 switch.
This upload mechanism requires that the switch is put into the serial download mode.
In order to do this the DIP-switches 1 and 2 needs to be off. This is the DIP-switch
on the carrier CRXS32G with two switches in total.

# S32G274ACONBOX KSZ9896 Switch

The platform S32G274ACONBOX is equipped with a switch KSZ9896. Starting with BSP32 the switch
is integrated into to Linux kernel using the Distributed Switch Architecture (DSA). The uplink
port 5 of the switch is connected to PFE1. Three ports of the switch are routed to external connectors.
These ports are port 0, 2, and 4 called sw00, sw02, and sw04.

# Notes for Software Developers

## Create Your Own Layer <span id="create-layer"/>

Before you start modifying recipes it is the best when you create your own layer which
is supposed to contain your changes.

    1. If not already done go to the build directory of the machine and call:
       # . SOURCE_THIS
       This configures and starts bitbake.
    2. Go to the sources/ directory and create your layer there:
       # bitbake-layers create-layer --priority 20 meta-mylayer<br>
       *Note 1:* because the priority of meta-microsys-auto is 10, it is recommended
       to choose a higher number as priority for your own layer (but less than 99).<br>
       *Note 2:* it is a good practice to prefix the layer name with "meta-".<br>
       *Note 3:* this is an optional step. You can remove the directory
       recipes-example in the directory meta-mylayer/.
    3. Go to the build directory of the machine and add your new layer with:
       # bitbake-layers add-layer ../sources/meta-mylayer

Now you are ready for modifying sources.

## Modifying U-Boot or Linux Kernel

You can modify the sources of U-Boot using the Yocto tool devtool. This task assumes that you have
created your own layer as described in the section [Create Your Own Layer](#create-layer).
Note that this works for other recipes, too. For example, if you plan to modify the Linux kernel
use  _linux-s32_  as recipe name.

    1. If not already done go to the build directory of the machine and call:

       # . SOURCE_THIS

       This configures and starts bitbake.

    2. Call devtool with the recipe for U-Boot as argument:

       # devtool modify u-boot-s32

       This creates a layer called "workspace" which contains the U-Boot
       sources as a GIT-repository. The sources can be found in the directory
       workspace/sources/u-boot-s32.

       Note: if you have done this step before, for example from a previous
       modification, then you have to call:

           # devtool modify --no-extract u-boot-s32

       with the additional option '--no-extract'. In this case devtool expects that
       the source tree already exists.

    3. Modify the sources with your favourite editor.
       Build your U-Boot with:

       # devtool build u-boot-s32

       You can find the results in workspace/sources/u-boot-s32/oe-workdir/image/boot.
       The resulting U-Boot images are:

           - u-boot.s32-sdcard
           - u-boot.s32-qspi

       (The results for the Linux kernel can be found in
       workspace/sources/linux-s32/oe-workdir/image/boot)

       If you now want to test your new U-Boot you can create a new boot image with:

       # devtool build-image microsys-image-auto

       You can find the output files in tmp/deploy/images/s32g274asbc2.

    4. Once you're done with your changes finish your work

       - go to workspace/sources/u-boot-s32 and commit your changes using 'git'

       - Update the recipe with:

           # devtool finish u-boot-s32 meta-mylayer

       This copies and saves your changes into meta-mylayer.

       If you plan to make more modifications to U-Boot it's better not to delete
       the source tree from the 'workspace' directory. Keep it.

       Note: If you think that you are done with all of your changes then you can remove
       the recipe from the workspace layer with:

       # devtool reset u-boot-s32

       This will  **erase all**  of your changes from workspace.

    5. Now you can rebuild your image including your changes with:

       # bitbake microsys-image-auto

Note: if you want to modify the Linux kernel configuration you can call:

       # bitbake -c menuconfig linux-s32

## Generating the Cross-Toolchain

For cross-development on the host you can generate and install a cross-toolchain
using Yocto build environment.

    # bitbake -c populate_sdk microsys-image-auto

This generates a self-extractor (shar) in the directory tmp/deploy/sdk called
`microsys-auto-glibc-x86_64-aarch64-toolchain-<version>.sh` which can be installed on
your build host.

## Exchange the Boot Sequence

Important note: the described method is only intended for development or evaluation.
It is **not** recommended for production or for use in the field!

The following steps exchange the boot media:

    - boot into U-Boot

    - type:
        => i2c dev 0
        => i2c read 50 0.1 4 90000000
        => i2c read 56 0.1 4 90000010

    - optionally, you can check the contents with:
        => md.b 90000000
        90000000: 40 01 0f 00 00 00 00 00 00 00 00 00 00 00 00 00    @...............
        90000010: 00 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00    ................
        Note: this is sample output from S32G274ASBC2

    - now re-program the EEPROMs:
        => i2c write 90000010 50 0.1 4 -s
        => i2c write 90000000 56 0.1 4 -s

Note: do the above steps in one go without reset or power-cycle!

# Start Firmware on CM7_0 Core

It is assumed that you have already built a firmware image for CM7_0 core, for example with
S32 Design Studio.
You can load and start the image in U-Boot with the following commands:

    => setenv fw_file m7_example.bin
    => setenv fw_start_addr 0x34300000
    => setenv fw_entry_addr 0x34501000
    => setenv load_fw 'dcache off; initsram 0x34100000 0x700000; ext4load mmc 0:1 ${fw_start_addr} ${fw_file}; startm7 ${fw_entry_addr}'
    => run load_fw

Note that the addresses in `fw_start_addr` and `fw_entry_addr` can differ
depending of what you have compiled in.
