# Copyright (C) 2020-2021 MicroSys Electronics GmbH
# Author: Kay Potthoff <kay.potthoff@microsys.de>

inherit microsys

FILESEXTRAPATHS_prepend := "${THISDIR}/${PN}:"
PATCHTOOL = "git"
PATCH_COMMIT_FUNCTIONS = "1"

#BRANCH = "release/bsp31.0-5.10.41-rt"
SRCREV = "47a49b91d4d439be9dee1ee62fc8f1652113bbc6"

python do_patch_prepend() {
    if d.getVar('PATCHTOOL') == 'git':
        srcsubdir = d.getVar('S')
        bb.process.run('git config am.threeWay true', cwd=srcsubdir)
        bb.process.run('git config am.keepcr true', cwd=srcsubdir)
        bb.process.run('git config apply.whitespace fix', cwd=srcsubdir)
        bb.process.run('git config apply.ignoreWhitespace change', cwd=srcsubdir)
}

PLATFORM ?= "${MACHINE}"
ITS_FILE ?= "${@microsys_get_its_file(d)}"

SCMVERSION = "n"

do_install[depends] += "u-boot-tools-native:do_populate_sysroot"

INSTALL_MOD_PATH = "${D}"
EXTRA_OEMAKE += 'INSTALL_MOD_PATH="${D}"'

KERNEL_ALT_IMAGETYPE += "Image.gz"

KERNEL_MODULE_AUTOLOAD_mpxs32g274a += "\
    nxp \
"

LINUX_TOOLS ?= "gpio leds spi usb"

do_compile_append() {

    its_file=${ITS_FILE} # ${S}/kernel-${PLATFORM}.its

    if [ ! -f ${its_file} ]; then
        its_file = ${S}/kernel-mpxls10xx.its
    fi
    default="            default = \"${ITS_DEVICETREE_CFG}\";"
    if [ -n "${ITS_DEVICETREE_CFG}" ]; then
        sed -e "s/^ *default *=.*$/${default}/g" ${its_file} > ${B}/kernel-${PLATFORM}.its
    else
        cp ${its_file} ${B}/kernel-${PLATFORM}.its
    fi

    # Build Linux tools

    cd ${S}
    oe_runmake -C tools/ ${LINUX_TOOLS}
}

do_install_append() {
    if [ -f "arch/${ARCH}/boot/Image" ]; then
        install -m 0644 arch/${ARCH}/boot/Image ${D}/boot/Image-${KERNEL_VERSION}
        ln -sf  Image-${KERNEL_VERSION} ${D}/boot/Image
    fi
    if [ -f "arch/${ARCH}/boot/Image.gz" ]; then
        install -m 0644 arch/${ARCH}/boot/Image.gz ${D}/boot/Image-${KERNEL_VERSION}.gz
        ln -sf  Image-${KERNEL_VERSION}.gz ${D}/boot/Image.gz
    fi
    mkimage -f kernel-${PLATFORM}.its fitImage.itb
    install -m 0644 fitImage.itb ${D}/boot/fitImage-${KERNEL_VERSION}.itb
    ln -sf fitImage-${KERNEL_VERSION}.itb ${D}/boot/fitImage.itb

    bbdebug 1 "Installing fitImage-${KERNEL_VERSION}.itb\n"
    install -d ${DEPLOY_DIR_IMAGE}
    install -m 0644 ${D}/boot/fitImage-${KERNEL_VERSION}.itb ${DEPLOY_DIR_IMAGE}/fitImage-${KERNEL_VERSION}.itb
    ln -sf fitImage-${KERNEL_VERSION}.itb ${DEPLOY_DIR_IMAGE}/fitImage.itb

    # Install Linux tools
    # (the Makefile in tools/ doesn't install the tools with root ownership
    # therefore, we're doing this manually here)

    install -d -o root -g root ${D}/usr/bin
    install -m 0755 -o root -g root ${S}/tools/gpio/lsgpio ${D}/usr/bin
    install -m 0755 -o root -g root ${S}/tools/gpio/gpio-event-mon ${D}/usr/bin
    install -m 0755 -o root -g root ${S}/tools/gpio/gpio-hammer ${D}/usr/bin
    install -m 0755 -o root -g root ${S}/tools/leds/uledmon ${D}/usr/bin
    install -m 0755 -o root -g root ${S}/tools/usb/testusb ${D}/usr/bin
    install -m 0755 -o root -g root ${S}/tools/spi/spidev_test ${D}/usr/bin

    # pcitest is now covered by recipe kernel-pcitest
    #install -m 0755 -o root -g root ${S}/tools/pci/pcitest ${D}/usr/bin
}

do_deploy_append() {
    bbdebug 1 "Deploying fitImage-${KERNEL_VERSION}.itb\n"
    install -d ${DEPLOY_DIR_IMAGE}
    install -m 0644 ${D}/boot/fitImage-${KERNEL_VERSION}.itb ${DEPLOY_DIR_IMAGE}/fitImage-${KERNEL_VERSION}.itb
    ln -sf fitImage-${KERNEL_VERSION}.itb ${DEPLOY_DIR_IMAGE}/fitImage.itb
}

FILES_${KERNEL_PACKAGE_NAME}-image += "/boot/Image*"
FILES_${KERNEL_PACKAGE_NAME}-image-fitimage += "/boot/fitImage*"

PACKAGES += "${PN}-images"
FILES_${PN}-images += "/boot"
#COMPATIBLE_MACHINE = "(s32)"

FILES_${PN} += "/boot/fitImage*"

PACKAGES += "${PN}-tools"
FILES_${PN}-tools += " \
   /usr/bin \
"
INSANE_SKIP_${PN}-tools = "ldflags debug-files"

SRC_URI += "file://0001-gpio-fxl6408-added-driver.patch;name=patch0001"
SRC_URI += "file://0002-hwmon-lm90-added-IRQ-support.patch;name=patch0002"
SRC_URI += "file://0003-net-stmicro-check-for-MDIO-bus.patch;name=patch0003"
SRC_URI += "file://0004-phy-marvell-support-for-88E1548P-and-88Q2112.patch;name=patch0004"
SRC_URI += "file://0005-net-phy-dp83867-changes-from-LSDK.patch;name=patch0005"
SRC_URI += "file://0006-drivers-ksz9477-added-KSZ9896.patch;name=patch0006"
SRC_URI += "file://0007-net-pcs-fsl-s32gen1-xpcs-enabled-SGMII-AN.patch;name=patch0007"
SRC_URI += "file://0008-phy-freescale-phy-fsl-s32gen1-serdes.c-fixed-mixup-X.patch;name=patch0008"
SRC_URI += "file://0009-net-phy-phylink-disabled-AN-for-2.5Gbs.patch;name=patch0009"
SRC_URI += "file://0010-net-ethernet-dwmac-s32cc.c-set-AN-completion.patch;name=patch0010"
SRC_URI += "file://0011-s32g274asbc-added-configuration.patch;name=patch0011"
SRC_URI += "file://0012-s32g274asbc2-added-configuration.patch;name=patch0012"
SRC_URI += "file://0013-s32g274asbc3-added-configuration.patch;name=patch0013"
SRC_URI += "file://0014-s32g274aconbox-added-platform.patch;name=patch0014"
SRC_URI += "file://0015-dts-mpxs32g274a-added-module-device-tree.patch;name=patch0015"
SRC_URI += "file://0016-s32g274a-added-make-of-device-trees.patch;name=patch0016"
SRC_URI += "file://0017-localversion-5.0.patch;name=patch0017"
SRC_URI += "file://0018-net-pcs-fsl-s32gen1-xpcs.c-fixed-typo.patch;name=patch0018"
SRC_URI += "file://0019-s32g274aconbox-dts-enabled-KSZ-switch.patch;name=patch0019"
SRC_URI += "file://0020-Committing-changes-from-do_symlink_kernsrc.patch;name=patch0020"
SRC_URI += "file://0021-net-dsa-added-interface-mode-to-port.patch;name=patch0021"
SRC_URI += "file://0022-dsa-microchip-adjust-interface-mode.patch;name=patch0022"
SRC_URI += "file://0023-s32g274aconbox-dts-configuration-of-compat_interface.patch;name=patch0023"
SRC_URI += "file://0024-localversion-5.0.1.patch;name=patch0024"
SRC_URI += "file://0025-dts-s32g274asbc2-include-of-mpxs32g274a.dtsi.patch;name=patch0025"
SRC_URI += "file://0026-dts-mpxs32g274a-added-ethernet-aliases.patch;name=patch0026"
SRC_URI += "file://0027-dts-mpxs32g274a-disable-GPIO-IRQs.patch;name=patch0027"
SRC_URI += "file://0028-localversion-5.0.2.patch;name=patch0028"
SRC_URI += "file://0029-dts-s32g274aconbox-fixed-UART1-2-pin-config.patch;name=patch0029"
SRC_URI += "file://0030-dts-s32g274asbc3-set-I2C2-clock-to-100KHz.patch;name=patch0030"
SRC_URI += "file://0031-drivers-gpio-gpio-fxl6408-added-set_config.patch;name=patch0031"
SRC_URI += "file://0032-net-phy-dp83867-retrigger-SGMII-AN-when-link-change.patch;name=patch0032"
SRC_URI += "file://0033-net-phy-dp83867-retrigger-SGMII-AN-only-if-enabled.patch;name=patch0033"
SRC_URI += "file://0034-s32g274asbc33-added-platform.patch;name=patch0034"
SRC_URI += "file://0035-localversion-5.0.6.patch;name=patch0035"

SRC_URI[patch0001.md5sum] = "f720bd83f3b72fff76a908ae739f1fde"
SRC_URI[patch0002.md5sum] = "da1ecdeb92c91b6ec83f0f877bd707c4"
SRC_URI[patch0003.md5sum] = "9a4815e483ef09595e797c0fff28aa05"
SRC_URI[patch0004.md5sum] = "1ac3cb873b95908f367b520b7720ee2a"
SRC_URI[patch0005.md5sum] = "e00b54a65fb2ebfbf7428e4eea0845bc"
SRC_URI[patch0006.md5sum] = "4bd3afa6b521ae43f8936ad579b3400c"
SRC_URI[patch0007.md5sum] = "41ba4007f1eb80f6e6b853654715884a"
SRC_URI[patch0008.md5sum] = "eb4fc629ea993b2a2ccabb3851bb69ed"
SRC_URI[patch0009.md5sum] = "f8bcaac059f53dc3d8d11a8f4b14bfd3"
SRC_URI[patch0010.md5sum] = "689c2b6e52ac22906e9c6b4d3e3364a7"
SRC_URI[patch0011.md5sum] = "6bee90e6660569497be2936e7c5be419"
SRC_URI[patch0012.md5sum] = "566b918e42ce2bc7c14b788278a10984"
SRC_URI[patch0013.md5sum] = "687a47db83bacd3bfb16d87381f55c49"
SRC_URI[patch0014.md5sum] = "c89b7defd1759ac9fd3795aac9330fda"
SRC_URI[patch0015.md5sum] = "5cf831a51407c73adbc1acbd81f1acc8"
SRC_URI[patch0016.md5sum] = "828a17751ec5402adb0e56c702b12e57"
SRC_URI[patch0017.md5sum] = "e2b97f13ccdb949d7d1a4248afb88395"
SRC_URI[patch0018.md5sum] = "e5042b8dbcc2bb69d2c2a52a933e9798"
SRC_URI[patch0019.md5sum] = "d31add2b4d313ebdeac88fa098ef2e99"
SRC_URI[patch0020.md5sum] = "b5ccc59c4b22b846c5b09bbb151e7e8c"
SRC_URI[patch0021.md5sum] = "7b8cbf5fd13b747a66976b0421b66e70"
SRC_URI[patch0022.md5sum] = "d3af16d178bea27a8c49fcae2bd5c62c"
SRC_URI[patch0023.md5sum] = "d87aaa701dd6c7cf64dda08d7fc034b7"
SRC_URI[patch0024.md5sum] = "9b695135e1a8046ba13c279bfca5d5bf"
SRC_URI[patch0025.md5sum] = "0941bea112a5216dd83b66138380df1a"
SRC_URI[patch0026.md5sum] = "f58fc719b363d5a61693cbd709f73879"
SRC_URI[patch0027.md5sum] = "93d86dad885b1ff181edfcdade95e49f"
SRC_URI[patch0028.md5sum] = "9bb0cbaea7d64e3d38077e8e3f21aaa0"
SRC_URI[patch0029.md5sum] = "8ded4516a939d17959e9b26bdfdfd088"
SRC_URI[patch0030.md5sum] = "522e46ba9151a82ce0c7e2f8a98dcfd7"
SRC_URI[patch0031.md5sum] = "62ab12bd8dc00aac2055303141d9b7d2"
SRC_URI[patch0032.md5sum] = "72f9deaf88be2920ea8a366d1675614d"
SRC_URI[patch0033.md5sum] = "5b7af81f1c923d6dcff48d546442313e"
SRC_URI[patch0034.md5sum] = "b5df6a7fa7ef5b356cb02b8a2b7c395c"
SRC_URI[patch0035.md5sum] = "90379f51adf530503f886193c3dbaa01"
