# Copyright (C) 2020-2021 MicroSys Electronics GmbH
# Author: Kay Potthoff <kay.potthoff@microsys.de>

require recipes-fsl/images/fsl-image-auto.bb
inherit deploy setuptools

IMAGE_INSTALL_append_mpxs32g274a = "${@bb.utils.contains('DISTRO_FEATURES', 'pfe', ' pfe ', '', d)}"
IMAGE_INSTALL_append_mpxs32g274a = "${@bb.utils.contains('DISTRO_FEATURES', 'sja1110', ' sja1110 ', '', d)}"
IMAGE_INSTALL_append = " vlan "
IMAGE_INSTALL_append_s32g274aconbox = " pps-tools "
#IMAGE_INSTALL_append_s32g274aconbox = " ksz989x-setup "

MACHINE_ESSENTIAL_EXTRA_RRECOMMENDS_mpxs32g274a += "kernel-module-pfe"
MACHINE_ESSENTIAL_EXTRA_RRECOMMENDS_mpxs32g274a += "${@bb.utils.contains('DISTRO_FEATURES', \
    'sja1110', 'kernel-module-sja1110', '', d)}"
MACHINE_ESSENTIAL_EXTRA_RRECOMMENDS_mpxs32g274a += "${@bb.utils.contains('DISTRO_FEATURES', \
    'hailo', 'kernel-module-hailo-pci', '', d)}"

MICROSYS_IMAGE_EXTRA_INSTALL += " \
    parted e2fsprogs-resize2fs bashprofile \
    linux-s32-tools libgpiod libgpiod-tools libgpiod-python libgpiodcxx \
    stressapptest \
"

MICROSYS_IMAGE_EXTRA_INSTALL += "${@bb.utils.contains('DISTRO_FEATURES', \
                                   'systemd', 'systemd pm-utils', '', d)}"

IMAGE_INSTALL += "${MICROSYS_IMAGE_EXTRA_INSTALL}"

IMAGE_INSTALL_remove = "kernel-devicetree"
IMAGE_INSTALL_remove = "gnuplot"
#IMAGE_INSTALL_remove = "u-boot-environment"
IMAGE_INSTALL_remove = "kernel-pcitest"
IMAGE_INSTALL_remove_gen1 = "aquantia-firmware-utility"

IMAGE_INSTALL_append_s32g274asbc2 = " s32g274aconbox-utils "
IMAGE_INSTALL_append_s32g274asbc3 = " s32g274aconbox-utils "

# Hailo support
# Recipes from meta-hailo-accelerator:
IMAGE_INSTALL_append = "${@bb.utils.contains('DISTRO_FEATURES', \
    'hailo', \
    ' hailo-pci hailo-firmware ', \
    '', d)}"

# Recipes from meta-hailo-libhailort:
#IMAGE_INSTALL_append = "${@bb.utils.contains('DISTRO_FEATURES', \
#    'hailo', \
#    ' hailortcli libhailort pyhailort pyhailort-shared ', \
#    '', d)}"
#IMAGE_INSTALL_append = "${@bb.utils.contains('DISTRO_FEATURES', \
#    'hailo', \
#    ' python3-distlib python3-verboselogs ', \
#    '', d)}"
#IMAGE_INSTALL_append = "${@bb.utils.contains('DISTRO_FEATURES', \
#    'hailo', \
#    ' libgsthailo ', \
#    '', d)}"

# Recipes from meta-hailo-tappas:
#IMAGE_INSTALL_append = "${@bb.utils.contains('DISTRO_FEATURES', \
#    'hailo', \
#    ' xtl hailo-post-processes libgsthailotools tappas-apps rapidjson xtensor cxxopts ', \
#    '', d)}"

# Temporary work-around (unable to find a match):
#IMAGE_INSTALL_remove = "inetutils-ping6"

SDCARD_ROOTFS = "${IMGDEPLOYDIR}/${IMAGE_BASENAME}-${MACHINE}.ext4"

DEPENDS += " parted-native u-boot "

DEPENDS += "arm-trusted-firmware"

do_image_ext4[depends] += "e2fsprogs-native:do_populate_sysroot"
do_image_ext3[depends] += "e2fsprogs-native:do_populate_sysroot"
do_image_sdcard[deptask] += "do_rootfs"
do_image_sdcard[deptask] += "do_image_ext4"
do_image_sdcard[deptask] += "do_image_qspi"
do_rootfs_kernelitb[deptask] += "virtual/kernel:do_deploy"
do_image_sdcard[depends] += "arm-trusted-firmware:do_deploy"
do_image_qspi[depends] += "arm-trusted-firmware:do_deploy"

IMAGE_FSTYPES_remove = "cpio.gz.u-boot"
IMAGE_FSTYPES_remove = "ext3"
IMAGE_FSTYPES += "ext4"
IMAGE_FSTYPES_append_mpxs32g274a = " qspi"

do_rootfs_kernelitb() {
    cp -a ${DEPLOY_DIR_IMAGE}/fitImage*.itb ${IMAGE_ROOTFS}/boot
    rm -f ${IMAGE_ROOTFS}/boot/vmlinux*
    rm -f ${IMAGE_ROOTFS}/boot/fsl-image-*
    rm -f ${IMAGE_ROOTFS}/boot/Image*
}
addtask rootfs_kernelitb after do_rootfs before do_image

do_rootfs_firmware() {
}

do_rootfs_firmware_mpxs32g274a() {
    # PFE firmware:
    # cp -a ${DEPLOY_DIR_IMAGE}/${SDCARDIMAGE_BOOT_EXTRA2_FILE} ${IMAGE_ROOTFS}

    # QSPI U-Boot:
    mkdir -p ${IMAGE_ROOTFS}/boot
    cp ${DEPLOY_DIR_IMAGE}/${FLASHIMAGE_UBOOT_FILE} ${IMAGE_ROOTFS}/boot
    # cp ${DEPLOY_DIR_IMAGE}/u-boot-${MACHINE}-default-environment.bin-* ${IMAGE_ROOTFS}/boot
}
addtask rootfs_firmware after do_rootfs before do_image
